@@ -1,22 +1,23 @@
-/* -copyright-
-#-# 
-#-# xsnow: let it snow on your desktop
-#-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021,2022,2023,2024 Willem Vermin
-#-# 
-#-# This program is free software: you can redistribute it and/or modify
-#-# it under the terms of the GNU General Public License as published by
-#-# the Free Software Foundation, either version 3 of the License, or
-#-# (at your option) any later version.
-#-# 
-#-# This program is distributed in the hope that it will be useful,
-#-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-#-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#-# GNU General Public License for more details.
-#-# 
-#-# You should have received a copy of the GNU General Public License
-#-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#-# 
+/* 
+   -copyright-
+# xsnow: let it snow on your desktop
+# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
+#              2019,2020,2021,2022,2023,2024 Willem Vermin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# 
+#-endcopyright-
  *
  */
 /*
@@ -51,6 +52,7 @@
 #include <X11/extensions/Xinerama.h>
 #include <ctype.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 #include <cairo-xlib.h>
 #include <gsl/gsl_version.h>
 #include <stdlib.h>
@@ -58,8 +60,14 @@
 #include <signal.h>
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
 #include "mygettext.h"
 
+//#ifdef assert
+//#undef assert
+//#define assert(x) {} 
+//#endif
+
 #include "xsnow-constants.h"
 
 #include "aurora.h"
@@ -109,8 +117,7 @@ int              SnowWinChanged = 1;
 cairo_t         *CairoDC = NULL;
 cairo_surface_t *CairoSurface = NULL;
 
-
-
+int lxdefound = False;
 // miscellaneous
 char       Copyright[] = "\nXsnow\nCopyright 1984,1988,1990,1993-1995,2000-2001 by Rick Jansen, all rights reserved, 2019,2020 also by Willem Vermin\n";
 
@@ -153,20 +160,29 @@ static void   SetWindowScale(void);
 static void   GetDesktopSession(void);
 static void   rectangle_draw(cairo_t *cr);
 static void   mybindtestdomain(void);
+static void   PrintAbsoluteCoordinates(void);
+static void   DisplayDimensions(void);
+static void   InitDisplayDimensions();
+static void   make_fullmaxscreen(void);
 
 // callbacks
-static int do_displaychanged(void *);
-static int do_draw_all(gpointer widget);
-static int do_event(void *);
-static int do_show_range_etc(void *);
-static int do_testing(void *);
-static int do_ui_check(void *);
-static int do_stopafter(void *);
-static int do_show_desktop_type(void *);
-static int do_display_dimensions(void *);
-static int do_drawit(void*);
+static int  do_displaychanged(void *);
+static int  do_write_flags(void *);
+static int  do_draw_all(gpointer widget);
+static int  do_event(void *);
+static int  do_show_range_etc(void *);
+static int  do_testing(void *);
+static int  do_ui_check(void *);
+static int  do_stopafter(void *);
+static int  do_show_desktop_type(void *);
+static int  do_display_dimensions(void *);
+static int  do_drawit(void *);
+static int  do_check_stop(void *);
+static int  do_full_trans(gpointer widget);
 static gboolean     on_draw_event(GtkWidget *widget, cairo_t *cr, gpointer user_data);
 
+static volatile int signal_caught = 0;
+static char *StopFile;
 /**********************************************************************************************/
 
 /* About some technicalities of this program
@@ -260,9 +276,12 @@ int main_c(int argc, char *argv[])
    signal(SIGINT,  SigHandler);
    signal(SIGTERM, SigHandler);
    signal(SIGHUP,  SigHandler);
+   signal(SIGUSR1, SigHandler);
    P("srand %d\n",(int)(fmod(wallcl()*1.0e6,1.0e8)));
    srand48((int)(fmod(wallcl()*1.0e6,1.0e8)));
 
+   const char *STOPFILE = ".xsnowstop";
+
    memset(&global,0,sizeof(global));
 
    XInitThreads();
@@ -271,56 +290,60 @@ int main_c(int argc, char *argv[])
    aurora_sem_init();
    birds_sem_init();
 
-   global.counter            = 0;
-   global.cpufactor          = 1.0;
-   global.WindowScale        = 1.0;
+   global.counter             = 0;
+   global.cpufactor           = 1.0;
+   global.WindowScale         = 1.0;
 
-   global.MaxSnowFlakeHeight = 0;
-   global.MaxSnowFlakeWidth  = 0;
-   global.FlakeCount         = 0;  /* # active flakes */
-   global.FluffCount         = 0;
+   global.MaxSnowFlakeHeight  = 0;
+   global.MaxSnowFlakeWidth   = 0;
+   global.FlakeCount          = 0;  /* # active flakes */
+   global.FluffCount          = 0;
 
-   global.SnowWin            = 0;
-   global.WindowOffsetX      = 0;
-   global.WindowOffsetY      = 0;
-   global.DesktopSession     = NULL;
-   global.CWorkSpace         = 0;
-   global.ChosenWorkSpace    = 0;
-   global.NVisWorkSpaces     = 1;
-   global.VisWorkSpaces[0]   = 0;
-   global.WindowsChanged     = 0;
-   global.ForceRestart       = 0;
+   global.SnowWin             = 0;
+   global.WindowOffsetX       = 0;
+   global.WindowOffsetY       = 0;
+   global.DesktopSession      = NULL;
+   global.CWorkSpace          = 0;
+   global.ChosenWorkSpace     = 0;
+   global.NVisWorkSpaces      = 1;
+   global.VisWorkSpaces[0]    = 0;
+   global.WindowsChanged      = 0;
+   global.ForceRestart        = 0;
 
-   global.FsnowFirst         = NULL;
-   global.MaxScrSnowDepth    = 0;
-   global.RemoveFluff        = 0;
+   global.FsnowFirst          = NULL;
+   global.MaxScrSnowDepth     = 0;
+   global.RemoveFluff         = 0;
 
-   global.SnowOnTrees        = NULL;
-   global.OnTrees            = 0;
+   global.SnowOnTrees         = NULL;
+   global.OnTrees             = 0;
 
-   global.moonX              = 1000;
-   global.moonY              = 80;
+   global.moonX               = 1000;
+   global.moonY               = 80;
 
-   global.Wind               = 0;
-   global.Direction          = 0;
-   global.WindMax            = 500.0;
-   global.NewWind            = 100.0;
+   global.Wind                = 0;
+   global.Direction           = 0;
+   global.WindMax             = 500.0;
+   global.NewWind             = 100.0;
 
-   global.HaltedByInterrupt  = 0;
-   global.Message[0]         = 0;
+   global.HaltedByInterrupt   = 0;
+   global.Message[0]          = 0;
 
-   global.SantaPlowRegion    = 0;
+   global.SantaPlowRegion     = 0;
 
-   global.DoCapella          = DOCAPELLA;
-
-   int i;
+   global.DoCapella           = DOCAPELLA;
+   global.time_to_write_flags = TRUE;
 
+   StopFile = (char*) malloc(sizeof(char)*(strlen(getenv("HOME"))+1+strlen(STOPFILE)+1));
+   assert(StopFile);
+   strcpy(StopFile,getenv("HOME"));
+   strcat(StopFile,"/");
+   strcat(StopFile,STOPFILE);
 
    InitFlags();
    // we search for flags that only produce output to stdout,
    // to enable to run in a non-X environment, in which case 
    // gtk_init() would fail.
-   for (i=0; i<argc; i++)
+   for (int i=0; i<argc; i++)
    {
       char *arg = argv[i];
       if(!strcmp(arg, "-h") || !strcmp(arg, "-help")) 
@@ -387,8 +410,9 @@ int main_c(int argc, char *argv[])
    //   -lang c
 
    Argv = (char**) malloc((argc+1)*sizeof(char**));
+   assert(Argv);
    Argc = 0;
-   for (i=0; i<argc; i++)
+   for (int i=0; i<argc; i++)
    {
       if ((strcmp("-screen",argv[i])==0) || (strcmp("-lang",argv[i])==0))
 	 i++; // found -screen or -lang
@@ -397,6 +421,7 @@ int main_c(int argc, char *argv[])
    }
    Argv[Argc] = NULL;
    GetDesktopSession();
+   assert(global.DesktopSession);
    if(!strncasecmp(global.DesktopSession,"bspwm",5))
    {
       printf(_("For optimal resuts, add to your %s:\n"),"bspwmrc");
@@ -423,8 +448,6 @@ int main_c(int argc, char *argv[])
       global.IsWayland = 0;
 
    gtk_init(&argc, &argv);
-   if (!Flags.NoConfig)
-      WriteFlags();
 
    if (Flags.BelowAllForce)
       Flags.BelowAll = 0;
@@ -471,7 +494,6 @@ int main_c(int argc, char *argv[])
 	 Flags.SnowColor2 = strdup(DefaultFlags.SnowColor2); 
       if (!ValidColor(Flags.BirdsColor))
 	 Flags.BirdsColor = strdup(DefaultFlags.BirdsColor); 
-      WriteFlags();
    }
 
 
@@ -524,6 +546,9 @@ int main_c(int argc, char *argv[])
       return 1;
    }
 
+   P("snowinx: %d, snowiny: %d, snowinwidth: %d, snowwinheight: %d\n",
+	 global.SnowWinX, global.SnowWinY, global.SnowWinWidth, global.SnowWinHeight);
+
 
 #define DOIT_I(x,d,v) OldFlags.x = Flags.x;
 #define DOIT_L(x,d,v) DOIT_I(x,d,v);
@@ -569,12 +594,16 @@ int main_c(int argc, char *argv[])
    loadmeasure_init();
    fallensnow_init();
 
-   add_to_mainloop(PRIORITY_DEFAULT, time_displaychanged,     do_displaychanged     );
-   add_to_mainloop(PRIORITY_DEFAULT, time_event,              do_event              );
-   add_to_mainloop(PRIORITY_DEFAULT, time_testing,            do_testing            );
-   add_to_mainloop(PRIORITY_DEFAULT, time_display_dimensions, do_display_dimensions );
-   add_to_mainloop(PRIORITY_HIGH,    time_ui_check,           do_ui_check           );
-   add_to_mainloop(PRIORITY_DEFAULT, time_show_range_etc,     do_show_range_etc     );
+   add_to_mainloop(PRIORITY_HIGH,    0.001,                      do_full_trans               );
+
+   add_to_mainloop(PRIORITY_DEFAULT, time_displaychanged,     do_displaychanged           );
+   add_to_mainloop(PRIORITY_DEFAULT, time_event,              do_event                    );
+   add_to_mainloop(PRIORITY_DEFAULT, time_testing,            do_testing                  );
+   add_to_mainloop(PRIORITY_DEFAULT, time_writeflags,         do_write_flags              );
+   add_to_mainloop(PRIORITY_DEFAULT, time_display_dimensions, do_display_dimensions       );
+   add_to_mainloop(PRIORITY_HIGH,    time_ui_check,           do_ui_check                 );
+   add_to_mainloop(PRIORITY_DEFAULT, time_show_range_etc,     do_show_range_etc           );
+   add_to_mainloop(PRIORITY_DEFAULT, time_check_stop,         do_check_stop               );
 
    if (Flags.StopAfter > 0)
       add_to_mainloop(PRIORITY_DEFAULT, Flags.StopAfter, do_stopafter);
@@ -585,6 +614,11 @@ int main_c(int argc, char *argv[])
 
    DoAllWorkspaces(); // to set global.ChosenWorkSpace
 
+   if (!Flags.NoConfig)
+      global.time_to_write_flags = TRUE;
+
+   PrintAbsoluteCoordinates();
+
    P("Entering main loop\n");
    // main loop
    gtk_main();
@@ -618,18 +652,23 @@ void set_below_above()
 {
    P("%d set_below_above %d\n",global.counter++,Flags.BelowAll);
    XWindowChanges changes;
+
    // to be sure: we do it in gtk mode and window mode
    if (Flags.BelowAll)
    {
-      if(TransA)setbelow(GTK_WINDOW(TransA));
+      if(TransA)
+	 setbelow(GTK_WINDOW(TransA));
       changes.stack_mode = Below;
-      if(global.SnowWin)XConfigureWindow(global.display,global.SnowWin,CWStackMode,&changes);
+      if(global.SnowWin)
+	 XConfigureWindow(global.display,global.SnowWin,CWStackMode,&changes);
    }
    else
    {
-      if(TransA)setabove(GTK_WINDOW(TransA));
+      if(TransA)
+	 setabove(GTK_WINDOW(TransA));
       changes.stack_mode = Above;
-      if(global.SnowWin)XConfigureWindow(global.display,global.SnowWin,CWStackMode,&changes);
+      if(global.SnowWin)
+	 XConfigureWindow(global.display,global.SnowWin,CWStackMode,&changes);
    }
 }
 
@@ -671,8 +710,7 @@ void GetDesktopSession()
 	 NULL
       };
 
-      int i;
-      for (i=0; desktops[i]; i++)
+      for (int i=0; desktops[i]; i++)
       {
 	 desktopsession = getenv(desktops[i]);
 	 if (desktopsession)
@@ -696,9 +734,10 @@ int StartWindow()
 
    global.Trans     = 0;
    global.xxposures = 0;
-   global.Desktop   = 0;
+   global.Desktop   = 0;  // well, it seems that it always becomes 1, so canditate for deletion
    global.UseDouble = 0;
    global.IsDouble  = 0;
+   global.UseClip   = 0;
    global.XscreensaverMode = 0;
 
    global.Rootwindow = DefaultRootWindow(global.display);
@@ -726,28 +765,33 @@ int StartWindow()
    }
    else
    {
+      P("Flags.Screen:%d\n",Flags.Screen);
       // default behaviour
       // try to create a transparent clickthrough window
       GtkWidget *gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_widget_set_can_focus(gtkwin, TRUE);
-      gtk_window_set_decorated(GTK_WINDOW(gtkwin), FALSE);
-      gtk_window_set_type_hint(GTK_WINDOW(gtkwin), GDK_WINDOW_TYPE_HINT_POPUP_MENU);
+      //gtk_widget_set_can_focus(gtkwin, TRUE);
+      //gtk_window_set_decorated(GTK_WINDOW(gtkwin), FALSE);
+      //gtk_window_set_type_hint(GTK_WINDOW(gtkwin), GDK_WINDOW_TYPE_HINT_POPUP_MENU);
 
 
       GdkWindow *gdkwin; (void)gdkwin;
-      int rc = make_trans_window(global.display, gtkwin,
+      int rc = make_trans_window(
+	    global.display, 
+	    gtkwin,
 	    Flags.Screen,        // full screen or xinerama
 	    Flags.AllWorkspaces, // sticky 
 	    Flags.BelowAll,      // below
-	    1,                   // dock
+				 //1,                   // dock
+	    0,                   // dock  see "IGNORE_ME" for xpenguins and others
 	    NULL,                // gdk_window
 	    &xwin,               // x11_window
 	    &wantx,              // make_trans_window tries to place the window here,
 				 //                      but, depending on window manager that does not always succeed
 	    &wanty
 	    );
-      if (rc)
+      if (rc) // we have a transparent snow window
       {
+	 // SnowWinWidth etc are determined in DisplayDimensions, called by do_display_dimensions
 	 global.Trans            = 1;
 	 global.IsDouble         = 1;
 	 global.Desktop          = 1;
@@ -758,16 +802,20 @@ int StartWindow()
 	 P("calling set_below_above\n");
 	 set_below_above();
 	 global.SnowWin = xwin;
-	 printf(_("Using transparent window\n"));
+
+	 int w,h;
+	 gtk_window_get_size(GTK_WINDOW(TransA),&w,&h);
+	 global.SnowWinWidth = w;
+	 global.SnowWinHeight = h;
+	 printf(_("Using transparent window: %dx%d\n"),w,h);
 	 P("wantx, wanty: %d %d\n",wantx,wanty);
 	 if(!strncasecmp(global.DesktopSession,"fvwm",4) ||
 	       !strncasecmp(global.DesktopSession,"lxqt",4))
 	 {
 	    printf(_("The transparent snow-window is probably not click-through, alas..\n"));
 	 }
-
       }
-      else
+      else  // we don't have a transparent snow window
       {
 	 global.Desktop  = 1;
 	 X11cairo = 1;
@@ -780,15 +828,22 @@ int StartWindow()
 	 // if envvar DESKTOP_SESSION == LXDE, search for window with name pcmanfm
 	 // largest_window_with_name uses name argument as regex, hence the '^' and '$'.
 	 // Note that the name is still case-insensitive
+
+	 int x,y;
+	 xinerama(global.display, Flags.Screen, &x, &y, NULL, NULL);
+	 P("Screen: %d x: %d y: %d\n",Flags.Screen, x, y);
+
+	 // In LXDE, there are many windows named pcmanfm
 	 if (!strncmp(global.DesktopSession,"LXDE",4) && 
-	       (xwin = largest_window_with_name(global.xdo,"^pcmanfm$"))
+	       (xwin = largest_window_with_name(global.xdo,"^pcmanfm$",x,y))
 	    )
 	 {
+	    lxdefound = True;
 	    printf(_("LXDE session found, using window 'pcmanfm'.\n"));
 	    P("lxdefound: %d %#lx\n",lxdefound,*xwin);
 	 }
 	 else if (
-	       (xwin = largest_window_with_name(global.xdo,"^Desktop$"))
+	       (xwin = largest_window_with_name(global.xdo,"^Desktop$",x,y))
 	       )
 	 {
 	    printf(_("Using window 'Desktop'.\n"));
@@ -800,13 +855,18 @@ int StartWindow()
 	 }
 	 global.SnowWin = xwin;
 	 int winw,winh;
+	 P("global.Desktop: %d\n",global.Desktop);
 	 if (Flags.Screen >=0 && global.Desktop)
+	 {
 	    xinerama(global.display,Flags.Screen,&wantx,&wanty,&winw,&winh);
+	    P("wantx: %d, wanty: %d, winw: %d, winh: %d\n",wantx, wanty, winw, winh);
+	 }
       }
    }
 
    if(X11cairo)
    {
+      P("X11cairo lxdefound: %d\n",lxdefound);
       HandleX11Cairo();
       drawit_id = add_to_mainloop1(PRIORITY_HIGH, time_draw_all, do_drawit, CairoDC);
       global.WindowOffsetX = 0;
@@ -833,15 +893,21 @@ int StartWindow()
       xdo_move_window(global.xdo,global.SnowWin,wantx,wanty);
       P("wantx wanty: %d %d\n",wantx,wanty);
    }
+
    xdo_wait_for_window_map_state(global.xdo,global.SnowWin,IsViewable);
+
+   if (global.Trans)
+      SetProperty(global.display,global.SnowWin,ignore_atom,"xsnow");
+
+   P("SnowWinWidth: %d SnowWinHeight: %d\n", global.SnowWinWidth, global.SnowWinHeight);
    InitDisplayDimensions();
 
    global.SnowWinX = wantx;
    global.SnowWinY = wanty;
 
-   printf(_("Snowing in %#lx: %s %d+%d %dx%d\n"),global.SnowWin,SnowWinName,global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
    PrevW = global.SnowWinWidth;
    PrevH = global.SnowWinHeight;
+   P("PrevW: %d PrevH: %d\n",PrevW,PrevH);
 
    P("woffx: %d %d\n",global.WindowOffsetX,global.WindowOffsetY);
 
@@ -860,8 +926,28 @@ int StartWindow()
    return TRUE;
 }
 
+void PrintAbsoluteCoordinates()
+{
+
+   // we did our best to place the snow window, but you never know what
+   // the window manager is cooking for us. So we ask politely the
+   // whereabouts of the snowwindow:
+
+   if (global.Trans){
+      //XWindowAttributes xwa;
+      //XGetWindowAttributes(global.display, global.SnowWin, &xwa );
+      int w,h;
+      gtk_window_get_size(GTK_WINDOW(TransA),&w,&h);
+      printf(_("Snowing in %#lx: %s size: %dx%d\n"),global.SnowWin,SnowWinName, 
+	    w,h);
+   }
+
+   fflush(stdout);
+}
+
 int HandleX11Cairo()
 {
+   P("Entering HandleX11Cairo: SnowWin: 0x%lx\n",global.SnowWin);
    unsigned int w,h;
    xdo_get_window_size(global.xdo, global.SnowWin, &w, &h);
    Visual *visual = DefaultVisual(global.display,DefaultScreen(global.display));
@@ -882,14 +968,17 @@ int HandleX11Cairo()
       if (CairoSurface)
 	 cairo_surface_destroy(CairoSurface);
       CairoSurface = cairo_xlib_surface_create(global.display, backBuf, visual, w, h);
+      // TODO do not use w,h but clipped w and h
       global.UseDouble = 1;
       global.IsDouble  = 1;
-      printf(_("Using double buffer: %#lx. %dx%d\n"),backBuf,w,h);
+      printf(_("Using double buffer: %#lx %dx%d\n"),backBuf,w,h);
       rcv = TRUE;
    }
 #endif
    if(!dodouble)
    {
+      if (CairoSurface)
+	 cairo_surface_destroy(CairoSurface);
       CairoSurface = cairo_xlib_surface_create(global.display, global.SnowWin, visual, w, h);
       printf(_("NOT using double buffering:"));
       if (Flags.UseDouble)
@@ -906,27 +995,97 @@ int HandleX11Cairo()
 
    CairoDC = cairo_create(CairoSurface);
    cairo_xlib_surface_set_size(CairoSurface,w,h);
+   P("setting snowwinwidth\n");
    global.SnowWinWidth  = w;
    global.SnowWinHeight = h;
+
+   // We are not able to create a transparent, click-through window so we have to paint
+   // in an existing window.
+   //
+   // Placement of the snow window:
+   // In multi-screen environments, it depends on your desktop manager how we
+   // should proceed.
+   // Simple X (also fvwm ant twm): There is one big (root) window, covering all screens.
+   //    In that case, we use cairo_rectangle and cairo_clip to abtain a drawing area
+   //    that starts at global.SnowWinX, global.SnowWinY, 
+   //       width: global.SnowWinWidth, height: global.SnowWinHeight
+   //
+   // Others, like LXDE, have already created windows, precisely covering
+   // the screens. We can check this by searching for a window that
+   // precisely covers the requested screen. The drawing coordinates of the
+   // must be 0,0 in that case.
+   //
+   // So, we find, using xinerama(), the coordinates and dimensions of the snow window.
+   //
+   // If there is a window with the same coordinates and sizes, we use that to draw in,
+   // and set the drawing coordinates to 0,0.
+   //
+   // Otherwize, we use cairo_rectangle and cairo_clip to define our drawing area.
+   //
+
+   int x1,y1,w1,h1;
+   xinerama(global.display, Flags.Screen, &x1, &y1, &w1, &h1);
+
+
+   WinInfo *windows;
+   int      nwin;
+
+   GetWindows(&windows,&nwin);
+   int ownwindow = False;
+   for (int i=0; i<nwin; i++)
+   {
+      WinInfo *w = &windows[i];
+      if (w->xa == x1 && w->ya == y1 && (int)w->w == w1 && (int)w->h == h1)
+      {
+	 P("match found: x1: %d y1: %d w1: %d h1: %d screen: %d id: %p\n",x1,y1,w1,h1,Flags.Screen,(void*)w->id);
+	 ownwindow = True;
+	 break;
+      }
+   }
+   if(windows)
+      free(windows);
+
    if (Flags.Screen >= 0 && global.Desktop)
    {
-      int winx, winy, winw, winh;
-      int rc = xinerama(global.display,Flags.Screen,&winx, &winy, &winw, &winh);
-      //if (rc && winx==0 && winy==0)
+      int winx=0, winy=0, winw, winh;
+      int rc = xinerama(global.display, Flags.Screen, &winx, &winy, &winw, &winh);
+      P("rc: %d winx: %d winy: %d winw: %d winh: %d\n",rc,winx,winy,winw,winh);
       if(rc)
       {
-	 global.SnowWinX      = winx;
-	 global.SnowWinY      = winy;
+	 if(ownwindow)
+	 {
+	    global.SnowWinX = 0;
+	    global.SnowWinY = 0;
+	 }
+	 else
+	 {
+	    global.SnowWinX = winx;
+	    global.SnowWinY = winy;
+	 }
+	 P("setting snowwinwidth\n");
 	 global.SnowWinWidth  = winw;
 	 global.SnowWinHeight = winh;
+	 PrevW                = winw;
+	 PrevH                = winh;
+      }
+      if(!ownwindow)
+      {
+	 cairo_rectangle(CairoDC,winx,winy,winw,winh);
+	 P("clipsnow winx: %d winy: %d snowwinx: %d snowwiny:%d snowwinwidth: %d snowwimheight: %d\n",winx, winy, global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
+	 cairo_clip(CairoDC);
+	 global.UseClip        = 1;
+	 global.SnowClipWidth  = winw;
+	 global.SnowClipHeight = winh;
+	 P("setting snowwinwidth\n");
+	 global.SnowWinWidth   = winw;      // will be overwitten by displaydimensions
+	 global.SnowWinHeight  = winh;     //  idem
+	 UpdateFallenSnowAtBottom();
       }
-      cairo_rectangle(CairoDC,global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
-      P("clipsnow %d %d %d %d\n",global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
-      cairo_clip(CairoDC);
    }
    return rcv;
 }
 
+
 void DoAllWorkspaces()
 {
    if(Flags.AllWorkspaces)
@@ -1009,7 +1168,8 @@ int do_ui_check(void *d)
 
    if (Flags.Changes > 0)
    {
-      WriteFlags();
+      //WriteFlags(1);
+      global.time_to_write_flags = TRUE;
       set_buttons();
       P("-----------Changes: %d\n",Flags.Changes);
    }
@@ -1018,6 +1178,33 @@ int do_ui_check(void *d)
    (void)d;
 }
 
+int do_write_flags(void *d)
+{
+   if (signal_caught)
+   {
+      printf("xsnow: caught signal %d\n",signal_caught);
+      fflush(stdout);
+      signal_caught = 0;
+   }
+   const int mc = 0;
+   // if one should wait a little longer before calling WriteFlags()
+   // one could set mc to 1, 2, ...
+   static int c = mc;
+   if (global.time_to_write_flags)
+   {
+      P("c: %d\n",c);
+      if (c > 0)
+      {
+	 c--;
+	 return TRUE;
+      }
+      WriteFlags(1);
+      global.time_to_write_flags = FALSE;
+      c = mc;
+   }
+   return TRUE;
+   (void)d;
+}
 
 int do_displaychanged(void *d)
 {
@@ -1027,12 +1214,10 @@ int do_displaychanged(void *d)
    (void)d;
    if (Flags.Done)
       return FALSE;
-   P("Trans: %d xxposures: %d Desktop: %d\n",
-	 global.Trans,   
-	 global.xxposures, global.Desktop);
+   P("Trans: %d xxposures: %d Desktop: %d\n", global.Trans,   global.xxposures, global.Desktop);
 
-   if (!global.Desktop)
-      return TRUE;
+   //if (!global.Desktop)
+   //  return TRUE;
 
    if (global.ForceRestart)
    {
@@ -1042,12 +1227,13 @@ int do_displaychanged(void *d)
    }
    else
    {
+      return TRUE; // TODO: disabling this section
       unsigned int w,h;
       Display* display = XOpenDisplay(Flags.DisplayName);
       Screen* screen   = DefaultScreenOfDisplay(display);
       w = WidthOfScreen(screen);
       h = HeightOfScreen(screen);
-      P("width height: %d %d %d %d\n",w,h,global.Wroot,global.Hroot);
+      P("w: %d, h: %d, Wroot: %d, Hroot: %d\n",w,h,global.Wroot,global.Hroot);
       if(global.Wroot != w || global.Hroot != h)
       {
 	 DoRestart = 1;
@@ -1098,13 +1284,28 @@ int do_event(void *d)
 	 } 
       }
    }  
-
    return TRUE;
 }
 
+// force stop if writeable file StopFile ($Home/.xsnowstop) exists
+int do_check_stop(void*dummy)
+{
+   (void)dummy;
+   if (access(StopFile,F_OK)) // 0: access, else no access
+      return TRUE;
+   if (access(StopFile,W_OK)) // 0: access, else no access
+      return TRUE;
+   if (unlink(StopFile) != 0)
+      return TRUE;
+   printf("Xsnow: halting because of existence of '%s'\n",StopFile);
+   Flags.Done = 1;
+   return FALSE;
+}
+
 void RestartDisplay()
 {
    P("Restartdisplay: %d W: %d H: %d\n",global.counter++,global.SnowWinWidth,global.SnowWinHeight);
+
    fflush(stdout);
    InitFallenSnow();
    init_stars();
@@ -1122,8 +1323,6 @@ void RestartDisplay()
       ClearScreen();
 }
 
-
-
 int do_show_range_etc(void *d)
 {
    if (Flags.Done)
@@ -1161,31 +1360,26 @@ int do_testing(void *d)
    (void)d;
    if (Flags.Done)
       return FALSE;
-   return TRUE;
-   int xret,yret;
-   unsigned int wret,hret;
-   xdo_get_window_location(global.xdo, global.SnowWin,&xret,&yret,NULL);
-   xdo_get_window_size(global.xdo, global.SnowWin, &wret, &hret);
-   P("%d wxh %d %d %d %d    %d %d %d %d \n",global.counter++,global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight,xret,yret,wret,hret);
-   P("WorkspaceActive, chosen: %d %ld\n",WorkspaceActive(),global.ChosenWorkSpace);
-   P("vis:");
-   int i;
-   for (i=0; i<global.NVisWorkSpaces; i++)
-      printf(" %ld",global.VisWorkSpaces[i]);
-   printf("\n");
-   printf("offsets: %d %d\n",global.WindowOffsetX,global.WindowOffsetY);
-   global.counter++;
 
-   for (i=0; i<global.NVisWorkSpaces; i++)
-      printf("%d: visible: %d %ld\n",global.counter,i,global.VisWorkSpaces[i]);
-
-   P("current workspace: %ld\n",global.CWorkSpace);
+   P("SnowWinX: %d SnowWinY: %d SnowWinWidth: %d SnowWinHeight: %d\n",global.SnowWinX,global.SnowWinY,global.SnowWinWidth, global.SnowWinHeight);
+   //int w,h;
+   //gtk_window_get_size(GTK_WINDOW(TransA),&w,&h); P("w: %d, h:%d\n",w,h);
+   P("SnowClipWidth: %d, SnowClipHeight: %d\n", global.SnowClipWidth, global.SnowClipHeight);
 
+   //PrintAbsoluteCoordinates();
    return TRUE;
 }
 
 void SigHandler(int signum)
 {
+   signal_caught = signum;
+   if (signum == SIGUSR1)
+   {
+      // Forbidden: printf in sighandler
+      //I("Caught SIGUSR1\n");
+      global.time_to_write_flags = TRUE;
+      return;
+   }
    global.HaltedByInterrupt = signum;
    Flags.Done        = 1;
 }
@@ -1237,6 +1431,7 @@ void drawit(cairo_t *cr)
    //   start of xsnow.
    // This is not harmful, but a bit annoying, so we do not draw anything 
    //   the first few times this function is called.
+
    if (counter*time_draw_all < 1.5)
    {
       counter++;
@@ -1336,22 +1531,60 @@ void SetWindowScale()
    P("WindowScale: %f\n",global.WindowScale);
 }
 
+int do_full_trans(gpointer widget)  // TODO can probably go
+{
+   (void) widget;
+   static int nscreens = -2;
+   static int counter = 0;
+   if(nscreens == -2)
+      nscreens = xinerama(global.display,-1,NULL,NULL,NULL,NULL);
+   P("do_full_trans\n");
+
+   // Make sure TransA is fullscreen or maximized and below or above as desired.
+   if(global.Trans)
+   {
+      if (Flags.Screen < 0 && nscreens > 1) 
+      {
+	 make_fullmaxscreen();
+      }
+      else
+	 gtk_window_maximize(GTK_WINDOW(TransA));
+      set_below_above();
+   }
+
+   if (counter++ > 10)
+      return FALSE;
+   return FALSE;
+}
+
 int do_display_dimensions(void *d)
 {
    (void)d;
    if (Flags.Done)
       return FALSE;
+
    if (!SnowWinChanged)
       return TRUE;
    SnowWinChanged = 0;
 
-   P("%d do_display_dimensions %dx%d %dx%d\n",global.counter++,global.SnowWinWidth,global.SnowWinHeight,PrevW,PrevH);
+   P("do_displaydimensions: calling DisplayDimensions()\n");
    DisplayDimensions();
    if (!global.Trans)
+   {
       HandleX11Cairo();
+   }
+   P("%d do_display_dimensions %dx%d %dx%d %d\n",global.counter++,global.SnowWinWidth,global.SnowWinHeight,PrevW,PrevH,global.Trans);
    if (PrevW != global.SnowWinWidth || PrevH != global.SnowWinHeight)
    {
-      printf(_("Window size changed, now snowing in in %#lx: %s %d+%d %dx%d\n"),global.SnowWin,SnowWinName,global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
+      {
+	 // we did our best to place the snow window, but you never know what
+	 // the window manager is cooking for us. So we ask politely the
+	 // whereabouts of the snowwindow:
+
+	 int x,y,w,h;
+	 GetAbsoluteCoordinates(global.display, global.SnowWin, &x, &y, &w, &h);
+	 printf(_("Window size changed, now snowing in in %#lx: %s %d+%d %dx%d\n"),global.SnowWin,SnowWinName,x,y,w,h);
+      }
       P("Calling RestartDisplay\n");
       RestartDisplay();
       PrevW = global.SnowWinWidth;
@@ -1364,11 +1597,15 @@ int do_display_dimensions(void *d)
 
 int do_draw_all(gpointer widget)
 {
+   static int counter = 0;
    if (Flags.Done)
       return FALSE;
    P("do_draw_all %d %p\n",global.counter++,(void *)widget);
+   if (global.Trans && counter < 4) // to be sure we repeat this 5 times
+      make_fullmaxscreen();  // a bit strange that we have to repeat this here ...
+   counter++;
 
-   // this will result in a call off on_draw_event():
+   // this will result in a call of on_draw_event():
    gtk_widget_queue_draw(GTK_WIDGET(widget));
    return TRUE;
 }
@@ -1377,13 +1614,14 @@ int do_draw_all(gpointer widget)
 // handle callbacks for things whose timings depend on cpufactor
 void HandleCpuFactor()
 {
+
    // re-add things whose timing is dependent on cpufactor
    if (Flags.CpuLoad <= 0)
       global.cpufactor = 1;
    else
       global.cpufactor = 100.0/Flags.CpuLoad;
 
-   P("handlecpufactor %f %f %d\n",oldcpufactor,cpufactor,counter++);
+   P("handlecpufactor %f %f %d\n",oldcpufactor,cpufactor,global.counter++);
    add_to_mainloop(PRIORITY_HIGH, time_init_snow , do_initsnow);  // remove flakes
 
    restart_do_draw_all();
@@ -1391,6 +1629,8 @@ void HandleCpuFactor()
 
 void restart_do_draw_all()
 {
+
+
    if (global.Trans)
    {
       if (draw_all_id)
@@ -1406,8 +1646,10 @@ void restart_do_draw_all()
       P("started do_drawit %d %p %f\n",drawit_id, (void *)CairoDC, time_draw_all);
    }
 }
+
 void rectangle_draw(cairo_t *cr)
 {
+
    const int lw = 8;
    cairo_save(cr);
    cairo_set_source_rgba(cr,1,1,0,0.5);
@@ -1415,6 +1657,7 @@ void rectangle_draw(cairo_t *cr)
    cairo_set_line_width(cr,lw);
    cairo_stroke(cr);
    cairo_restore(cr);
+
 }
 
 int do_stopafter(void *d)
@@ -1493,65 +1736,6 @@ void mybindtestdomain()
 
    // todo: naar restart of ui?
 
-   if(0)
-   {
-      if (strcmp(Flags.Language,"sys")) // if "sys" , do not change locale
-      {  // Language != sys
-	 char lc[100];
-	 if (!strcmp(Flags.Language,"en"))
-	    strcpy(lc,"en_US.UTF-8");
-	 else
-	 {
-	    strcpy(lc,Flags.Language);
-	    strcat(lc,"_");
-
-	    int i;
-	    int l = strlen(lc);
-	    for (i=0; i<(int)strlen(Flags.Language); i++)
-	       lc[l+i] = toupper(lc[i]);
-	    lc[l+i] = 0;
-
-	    strcat(lc,".UTF-8");
-	 }
-
-	 // in order for this to work, no gettext call must have been
-	 // made yet:
-
-	 //setenv("LC_ALL",lc,1);
-	 //setenv("LANG",lc,1);
-	 setenv("LANGUAGE",Flags.Language,1);
-	 P("LC_ALL: %s\n",getenv("LC_ALL"));
-	 P("LANG: %s\n",getenv("LANG"));
-	 P("LANGUAGE: %s\n",getenv("LANGUAGE"));
-      }
-      else // Language == sys
-      {
-	 unsetenv("LANGUAGE");
-	 unsetenv("LC_ALL");
-	 if(0)
-	 {
-	    char *l=getenv("LANG");
-	    if(l && !getenv("LANGUAGE"))
-	    {
-	       char *lang = strdup(l);
-	       P("lang: %s\n",lang);
-	       char *p = strchr(lang,'_');
-	       if(p)
-	       {
-		  *p = 0;  // TODO: wrong
-		  P("lang: %s\n",lang);
-		  setenv("LANGUAGE",lang,1);
-	       }
-	       free(lang);
-	    }
-	 }
-	 P("LC_ALL: %s\n",getenv("LC_ALL"));
-	 P("LANG: %s\n",getenv("LANG"));
-	 P("LANGUAGE: %s\n",getenv("LANGUAGE"));
-      }
-      //setenv("LANG","nl_NL.UTF8",1);
-      //setenv("LANGUAGE","nl",1);
-   }
 
    P("textdomain: %s\n",textdomain(NULL));
    P("bindtextdomain: %s\n",bindtextdomain(TEXTDOMAIN,NULL));
@@ -1559,3 +1743,94 @@ void mybindtestdomain()
 
 #endif
 }
+
+void DisplayDimensions()
+{
+   P("Displaydimensions\n");
+   Lock_fallen();
+   unsigned int w,h,b,d;
+   int x,y;
+
+   Window root;
+
+   int rc = XGetGeometry(global.display,global.SnowWin,&root, &x, &y, &w, &h, &b, &d); 
+
+   if (rc == 0)
+   {
+      P("Oeps\n");
+      I("\nSnow window %#lx has disappeared, it seems. I quit.\n",global.SnowWin);
+      Thanks();
+      exit(1);
+      return;
+   }
+
+   P("setting snowwinwidth Trans: %d\n",global.Trans);
+   if (global.Trans) 
+      gtk_window_get_size(GTK_WINDOW(TransA),(int *)&w,(int *)&h);
+   P("w: %d, h:%d\n",w,h);
+
+   if (global.UseClip)
+   {
+      global.SnowWinWidth  = global.SnowClipWidth;
+      global.SnowWinHeight = global.SnowClipHeight;
+   }
+   else
+   {
+      global.SnowWinWidth  = w;
+      global.SnowWinHeight = h;
+   }
+
+   P("DisplayDimensions: SnowWinX: %d Y:%d W:%d H:%d\n",global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
+
+   global.SnowWinBorderWidth = b;
+   global.SnowWinDepth       = d;
+
+   UpdateFallenSnowAtBottom();
+
+   RedrawTrees();
+
+   SetMaxScreenSnowDepth();
+   SetMaxWinSnowDepth();
+   if(!global.IsDouble)
+      ClearScreen();
+   Unlock_fallen();
+}
+
+void InitDisplayDimensions()
+{
+
+   unsigned int w,h;
+   int x,y;
+   xdo_get_window_location(global.xdo, global.Rootwindow, &x, &y,NULL);
+   xdo_get_window_size    (global.xdo, global.Rootwindow, &w, &h);
+
+   P("InitDisplayDimensions root: %p %d %d %d %d\n",(void*)global.Rootwindow,x,y,w,h);
+
+   global.Xroot = x;
+   global.Yroot = y;
+   global.Wroot = w;
+   global.Hroot = h;
+
+   DisplayDimensions();
+}
+
+void make_fullmaxscreen()
+{
+   P("make_maxfullscreen...\n");
+   if (Flags.Screen < 0)
+   {
+      static GdkWindow *gdkwin = NULL;
+      static int nmonitors = 0;
+      if (!nmonitors)
+	 nmonitors = xinerama(global.display, -1, NULL, NULL, NULL, NULL);
+      if (nmonitors == 1)  // return if number of monitors == 1
+	 return;
+      if (!gdkwin)
+	 gdkwin = gtk_widget_get_window(TransA);
+      gdk_window_set_fullscreen_mode(gdkwin,GDK_FULLSCREEN_ON_ALL_MONITORS);
+      gtk_window_fullscreen(GTK_WINDOW(TransA));
+   }
+   else
+      gtk_window_maximize(GTK_WINDOW(TransA));
+}
+
