@@ -1,22 +1,23 @@
-/* -copyright-
-#-# 
-#-# xsnow: let it snow on your desktop
-#-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021,2022,2023,2024 Willem Vermin
-#-# 
-#-# This program is free software: you can redistribute it and/or modify
-#-# it under the terms of the GNU General Public License as published by
-#-# the Free Software Foundation, either version 3 of the License, or
-#-# (at your option) any later version.
-#-# 
-#-# This program is distributed in the hope that it will be useful,
-#-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-#-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#-# GNU General Public License for more details.
-#-# 
-#-# You should have received a copy of the GNU General Public License
-#-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#-# 
+/* 
+   -copyright-
+# xsnow: let it snow on your desktop
+# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
+#              2019,2020,2021,2022,2023,2024 Willem Vermin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# 
+#-endcopyright-
 */
 /*
  * This works with EWHM/NetWM compatible X Window managers,
@@ -34,6 +35,7 @@
 #include <X11/Xatom.h>
 #include <string.h>
 #include <unistd.h>
+#include <assert.h>
 
 #include "xsnow-constants.h"
 
@@ -44,24 +46,25 @@
 
 #include "vroot.h"
 
+#if 0
 static void FindWindows(Display *display,Window window, long unsigned int *nwindows,Window **windows);
 static void FindWindows_r(Display *display,Window window,long unsigned int *nwindows,Window **windows);
 
-/* this one is not needed any more, but I keep the source */
+/* these functions are not needed any more, but I keep the source */
 void FindWindows(Display *display,Window window,long unsigned int *nwindows,Window **windows)
 {
    *nwindows = 0;
    *windows  = NULL;
    FindWindows_r(display,window,nwindows,windows);
-   int i;
-   for (i=0; i<(int)(*nwindows); i++)
+   for (int i=0; i<(int)(*nwindows); i++)
       P("window: %#lx\n",(*windows)[i]);
 }
+
+
 void FindWindows_r(Display *display,Window window,long unsigned int *nwindows,Window **windows)
 {
    Window root,parent,*children;
    unsigned int nchildren;
-   int i;
    static int generations = 0;
 
    XQueryTree(display,window,&root,&parent,&children,&nchildren);
@@ -75,7 +78,8 @@ void FindWindows_r(Display *display,Window window,long unsigned int *nwindows,Wi
 
       ++generations;
 
-      for (i = 0, child = children; i < (int)nchildren; ++i, ++child)
+      child = children;
+      for (int i = 0; i < (int)nchildren; ++i, ++child)
 	 FindWindows_r(display,*child,nwindows,windows);
 
       --generations;
@@ -84,6 +88,25 @@ void FindWindows_r(Display *display,Window window,long unsigned int *nwindows,Wi
 
    return;
 }
+#endif
+
+void GetAbsoluteCoordinates(Display *display, Window win, int *x, int *y, int *w, int *h)
+{
+   Window child;
+   XWindowAttributes xwa;
+   int xx,yy;
+   XTranslateCoordinates( display, win, DefaultRootWindow(display), 0, 0, &xx, &yy, &child );
+   XGetWindowAttributes( display, win, &xwa );
+   P( "window: %p x: %d y: %d w: %d h: %d\n", (void*)win, xx - xwa.x, yy - xwa.y, xwa.width, xwa.height );
+   if(x)
+      *x = xx - xwa.x;
+   if(y)
+      *y = yy - xwa.y;
+   if(w)
+      *w = xwa.width;
+   if(h)
+      *h = xwa.height;
+}
 
 long int GetCurrentWorkspace()
 {
@@ -187,6 +210,7 @@ int GetWindows(WinInfo **windows, int *nwin)
    static Atom atom_win_client_list;
    static Atom atom_win_workspace;
    static Atom atom_wm_state;
+   static Atom atom_ignore;
 
    static int firstcall = 1;
    if(firstcall)
@@ -204,6 +228,7 @@ int GetWindows(WinInfo **windows, int *nwin)
       atom_win_client_list      = XInternAtom(getdisplay, "_WIN_CLIENT_LIST"     ,False);
       atom_win_workspace        = XInternAtom(getdisplay, "_WIN_WORKSPACE"       ,False);
       atom_wm_state             = XInternAtom(getdisplay, "WM_STATE"             ,False);
+      atom_ignore               = XInternAtom(getdisplay, ignore_atom            ,False);
    }
 
    XGetWindowProperty(getdisplay, DefaultRootWindow(getdisplay), atom_net_client_list, 0, 1000000, False, 
@@ -235,11 +260,7 @@ int GetWindows(WinInfo **windows, int *nwin)
 	 XFree(children);
 	 children = NULL;
       }
-      if(0)
-      {
-	 FindWindows(getdisplay,RootWindow(getdisplay,DefaultScreen(getdisplay)),&nchildren,(Window **)&properties);
-      }
-      else
+
       {
 	 Window dummy;
 	 unsigned int n;
@@ -253,6 +274,8 @@ int GetWindows(WinInfo **windows, int *nwin)
    if(nchildren>0)
       (*windows) = (WinInfo *)malloc(nchildren*sizeof(WinInfo));
    WinInfo *w = (*windows);
+   if(w)
+      memset(w,0,sizeof(WinInfo));
    int k = 0;
 
    // and yet another check if window is hidden (needed e.g. in KDE/plasma after 'hide all windows')
@@ -281,9 +304,14 @@ int GetWindows(WinInfo **windows, int *nwin)
       }
    }
 
-   unsigned long i;
-   for (i=0; i<nchildren; i++)
+   //assert(w);
+   for (unsigned long i=0; i<nchildren; i++)
    {
+      // loop over all children of root
+      // all children are included in windows
+      // except those with depth=0
+      // sticky, dock, hidden and ignore are set as appropriate
+
       int x0,y0,xr,yr;
       unsigned int depth;
 
@@ -298,7 +326,8 @@ int GetWindows(WinInfo **windows, int *nwin)
       w->h  = winattr.height;
       depth = winattr.depth;
 
-      P("%d %#lx %d %d %d %d %d\n",counter++,w->id,x0,y0,w->w,w->h,depth);
+      //if (w->id == global.SnowWin)
+      P("#%d %#lx %d %d %d %d %d\n",global.counter++,w->id,x0,y0,w->w,w->h,depth);
       // if this window is showing nothing, we ignore it:
       if (depth == 0)
 	 continue;
@@ -317,6 +346,7 @@ int GetWindows(WinInfo **windows, int *nwin)
 	    AnyPropertyType, &type, &format, &nitems, &b, &properties);
       if(type != XA_CARDINAL)
       {
+	 // workspace number
 	 if(properties) XFree(properties);
 	 properties = NULL;
 	 XGetWindowProperty(getdisplay, w->id, atom_win_workspace, 0, 1, False, 
@@ -329,6 +359,18 @@ int GetWindows(WinInfo **windows, int *nwin)
       }
       else
 	 w->ws = 0;
+
+      // to be ignored by Atom ignore_atom?
+      XGetWindowProperty(getdisplay, w->id, atom_ignore, 0, (~0L), False,
+	    AnyPropertyType, &type, &format, &nitems, &b, &properties);
+      if(type == None)
+	 w->ignore = 0;
+      else
+	 w->ignore = 1;
+      if(properties)
+	 XFree(properties);
+      P("ignore #%lx %d %d\n",w->id,w->ignore,rc);
+
       // maybe this window is sticky:
       w->sticky = 0;
       properties = NULL;
@@ -337,8 +379,7 @@ int GetWindows(WinInfo **windows, int *nwin)
 	    AnyPropertyType, &type, &format, &nitems, &b, &properties);
       if (type == XA_ATOM)
       {
-	 int i;
-	 for(i=0; (unsigned long)i<nitems; i++)
+	 for(int i=0; (unsigned long)i<nitems; i++)
 	 {
 	    char *s = NULL;
 	    s = XGetAtomName(getdisplay,((Atom*)(void*)properties)[i]);
@@ -365,14 +406,13 @@ int GetWindows(WinInfo **windows, int *nwin)
 	    AnyPropertyType, &type, &format, &nitems, &b, &properties);
       if(format == 32)
       {
-	 int i;
-	 for(i=0; (unsigned long)i<nitems; i++)
+	 for(int i=0; (unsigned long)i<nitems; i++)
 	 {
 	    char *s = NULL;
 	    s = XGetAtomName(getdisplay,((Atom*)(void*)properties)[i]);
 	    if (!strcmp(s,"_NET_WM_WINDOW_TYPE_DOCK"))
 	    { 
-	       P("%#lx is dock %d\n",w->id, counter++);
+	       P("%#lx is dock #%d\n",w->id, global.counter++);
 	       w->dock = 1;
 	       if(s) XFree(s);
 	       break;
@@ -401,8 +441,7 @@ int GetWindows(WinInfo **windows, int *nwin)
 	       AnyPropertyType, &type, &format, &nitems, &b, &properties);
 	 if(format == 32)
 	 {
-	    unsigned long i;
-	    for (i=0; i<nitems; i++)
+	    for (unsigned long i=0; i<nitems; i++)
 	    {
 	       char *s = NULL;
 	       s = XGetAtomName(getdisplay,((Atom*)(void*)properties)[i]);
@@ -511,8 +550,7 @@ int GetWindows(WinInfo **windows, int *nwin)
 WinInfo *FindWindow(WinInfo *windows, int nwin, Window id)
 {
    WinInfo *w = windows;
-   int i;
-   for (i=0; i<nwin; i++)
+   for (int i=0; i<nwin; i++)
    {
       if (w->id == id)
 	 return w;
@@ -524,13 +562,13 @@ WinInfo *FindWindow(WinInfo *windows, int nwin, Window id)
 void printwindows(Display *dpy,WinInfo *windows, int nwin)
 {
    WinInfo *w = windows;
-   int i;
-   for (i=0; i<nwin; i++)
+   for (int i=0; i<nwin; i++)
    {
       char *name;
       XFetchName(dpy, w->id, &name);
       if (!name)
 	 name = strdup("No name");
+      assert(name);
       if (strlen(name)>20)
 	 name[20] = '\0';
       printf("id:%#10lx ws:%3ld x:%6d y:%6d xa:%6d ya:%6d w:%6d h:%6d sticky:%d dock:%d hidden:%d name:%s\n",
